﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using HIPS.CommonSchemas;
using HIPS.CommonSchemas.PatientIdentifier;
using HIPS.ConsentSchemas;
using HIPS.PcehrDataStore.Schemas.Enumerators;
using HIPS.PcehrSchemas;
using Nehta.VendorLibrary.PCEHR;

namespace HIPS.ServiceContracts.ServiceInterfaces
{
    /// <summary>
    /// Web service interface for the PCEHR service version 1.
    /// </summary>
    [WCFExtras.Wsdl.Documentation.XmlComments]
    [ServiceContract(Namespace = "http://schemas.HIPS/Services/2012/01", Name = "IPCEHRService")]
    public interface IPCEHRService
    {
        /// <summary>
        /// Requests emergency access to the patient's records on the PCEHR
        /// </summary>
        /// <param name="patientIdentifier">An identifier of the patient</param>
        /// <param name="user">The person responsible for this action</param>
        /// <returns>Gain PCEHR Access response.</returns>
        [OperationContract]
        GainPcehrAccessResponse GainAccessEmergency(PatientIdentifierBase patientIdentifier, UserDetails user);

        /// <summary>
        /// Request access with the access code being sent to the PCEHR
        /// </summary>
        /// <param name="patientIdentifier">An identifier of the patient.</param>
        /// <param name="AccessCode">The access code provided by the patient.</param>
        /// <param name="user">The person responsible for this action.</param>
        /// <returns>Gain PCEHR Access response.</returns>
        [OperationContract]
        GainPcehrAccessResponse GainAccessWithCode(PatientIdentifierBase patientIdentifier, string AccessCode, UserDetails user);

        /// <summary>
        /// Request for access without a code being required by PCEHR
        /// </summary>
        /// <param name="patientIdentifier">An identifier of the patient</param>
        /// <param name="user">The person responsible for this action</param>
        /// <returns>Gain PCEHR Access response.</returns>
        [OperationContract]
        GainPcehrAccessResponse GainAccessWithOutCode(PatientIdentifierBase patientIdentifier, UserDetails user);

        /// <summary>
        /// Return a list of uploaded and changed documents from a single document set.
        /// </summary>
        /// <param name="patientIdentifier">An identifier of the patient (MRN, State ID, PatientMasterId or IHI)</param>
        /// <param name="user">The health provider individual, interactive user or authorised employee responsible for this action</param>
        /// <param name="documentUniqueId">The document entry UUID (NOC requirements disallow using the document ID).</param>
        /// <returns>List of versions of the document.</returns>
        [OperationContract]
        ChangeHistoryViewResponse<PatientIdentifierBase> GetChangeHistoryView(PatientIdentifierBase patientIdentifier, UserDetails user, string documentUniqueId);

        /// <summary>
        /// This function is given the WS name of GetDocumentList.
        /// It returns all documents within the list without filtering the returned list
        /// </summary>
        /// <param name="patientIdentifier">An identifier of the patient</param>
        /// <param name="user">The health provider individual, interactive user or authorised employee responsible for this action</param>
        /// <returns>List of documents.</returns>
        [OperationContractAttribute(Name = "GetDocumentList")]
        DocumentListResponse<PatientIdentifierBase> GetDocumentList(PatientIdentifierBase patientIdentifier, UserDetails user);

        /// <summary>
        /// This function is given the WS name of GetDocumentListFilterDates.
        /// It returns all documents within the list with the ability to filter by start and end dates and times for the document creation and the healthcare service.
        /// </summary>
        /// <param name="patientIdentifier">An identifier of the patient</param>
        /// <param name="user">The person responsible for this action</param>
        /// <param name="creationTimeStart">Include documents created after this time.</param>
        /// <param name="creationTimeEnd">Include documents created before this time.</param>
        /// <param name="serviceTimeStart">Include documents for healthcare services after this time.</param>
        /// <param name="serviceTimeEnd">Include documents for healthcare services before this time.</param>
        /// <returns>List of documents.</returns>
        [OperationContractAttribute(Name = "GetDocumentListFilterDates")]
        DocumentListResponse<PatientIdentifierBase> GetDocumentList(PatientIdentifierBase patientIdentifier, UserDetails user, DateTime? creationTimeStart, DateTime? creationTimeEnd, DateTime? serviceTimeStart, DateTime? serviceTimeEnd);

        /// <summary>
        /// This function is given the WS name of GetDocumentListFilterStatusAndDates.
        /// It returns all documents within the list with the ability to filter by multiple statuses, start and end dates and times for document creation and the healthcare service.
        /// </summary>
        /// <param name="patientIdentifier">An identifier of the patient</param>
        /// <param name="user">The person responsible for this action</param>
        /// <param name="documentStatus">Include documents with this status.</param>
        /// <param name="creationTimeStart">Include documents created after this time.</param>
        /// <param name="creationTimeEnd">Include documents created before this time.</param>
        /// <param name="serviceTimeStart">Include documents for healthcare services after this time.</param>
        /// <param name="serviceTimeEnd">Include documents for healthcare services before this time.</param>
        /// <returns>List of documents.</returns>
        [OperationContractAttribute(Name = "GetDocumentListFilterStatusAndDates")]
        DocumentListResponse<PatientIdentifierBase> GetDocumentList(PatientIdentifierBase patientIdentifier, UserDetails user, IList<DocumentStatus> documentStatus, DateTime? creationTimeStart, DateTime? creationTimeEnd, DateTime? serviceTimeStart, DateTime? serviceTimeEnd);

        /// <summary>
        /// This function is given the WS name of GetDocumentListWithQuery.
        /// The DocumentQuery object can be used to create the full spectrum of searches on the Document List.
        /// </summary>
        /// <param name="patientIdentifier">An identifier of the patient</param>
        /// <param name="user">The person responsible for this action</param>
        /// <param name="docQuery">Detailed document query.</param>
        /// <returns>List of documents.</returns>
        [OperationContractAttribute(Name = "GetDocumentListWithQuery")]
        DocumentListResponse<PatientIdentifierBase> GetDocumentList(PatientIdentifierBase patientIdentifier, UserDetails user, DocumentQuery docQuery);

        /// <summary>
        /// This function is given the WS name of GetDocumentListActive.
        /// It returns all documents within the list that have the status as Approved
        /// </summary>
        /// <param name="patientIdentifier">An identifier of the patient</param>
        /// <param name="user">The person responsible for this action</param>
        /// <returns>List of documents.</returns>
        [OperationContractAttribute(Name = "GetDocumentListActive")]
        DocumentListResponse<PatientIdentifierBase> GetDocumentListActive(PatientIdentifierBase patientIdentifier, UserDetails user);

        /// <summary>
        /// This function is given the WS name of GetDocumentListActiveFilterDates.
        /// It returns all documents within the list that have the status as Approved and with the ability to filter by start and end dates and times for document creation and the healthcare service.
        /// </summary>
        /// <param name="patientIdentifier">An identifier of the patient</param>
        /// <param name="user">The person responsible for this action</param>
        /// <param name="creationTimeStart">Include documents created after this time.</param>
        /// <param name="creationTimeEnd">Include documents created before this time.</param>
        /// <param name="serviceTimeStart">Include documents for services after this time.</param>
        /// <param name="serviceTimeEnd">Include documents for services before this time.</param>
        /// <returns>List of documents.</returns>
        [OperationContractAttribute(Name = "GetDocumentListActiveFilterDates")]
        DocumentListResponse<PatientIdentifierBase> GetDocumentListActive(PatientIdentifierBase patientIdentifier, UserDetails user, DateTime? creationTimeStart, DateTime? creationTimeEnd, DateTime? serviceTimeStart, DateTime? serviceTimeEnd);

        /// <summary>
        /// Gets the status of a single pcehr queue operation for the specified PcehrMessageQueueId.
        /// </summary>
        /// <param name="pcehrMessageQueueId">Identifier of an item in the PCEHR Message Queue.</param>
        /// <param name="user">The user responsible for the action.</param>
        /// <returns>List of queued operations and indicator of success or failure.</returns>
        [OperationContract]
        IndividualOperationStatus GetIndividualOperationStatus(int pcehrMessageQueueId, UserDetails user);

        /// <summary>
        /// Get a document uploaded for a specific patient in a defined hospital
        /// </summary>
        /// <param name="user">Information to identify the person responsible for this action</param>
        /// <param name="sourceSystemSetId">The source system set identifier.</param>
        /// <param name="sourceSystemDocumentId">The source system document identifier.</param>
        /// <param name="patientIdentifier">The local, state or national patient identifier</param>
        /// <returns>
        /// Response containing a clinical document uploaded for a specific patient in a defined hospital
        /// </returns>
        [OperationContract]
        LocalClinicalDocumentResponse<PatientIdentifierBase> GetLocalUploadedDocument(UserDetails user, string sourceSystemSetId, string sourceSystemDocumentId, PatientIdentifierBase patientIdentifier);

        /// <summary>
        /// Gets the status of all operations for the specified patient and episode.
        /// </summary>
        /// <param name="patientIdentifier">The patient identifier</param>
        /// <param name="admissionDate">The admission date and time</param>
        /// <param name="user">The user</param>
        /// <returns>List of queued operations and indicator of success or failure.</returns>
        [OperationContract]
        OperationStatus GetOperationStatus(PatientIdentifierBase patientIdentifier, DateTime admissionDate, UserDetails user);

        /// <summary>
        /// Gets a List of queued Operations
        /// </summary>
        /// <param name="dateTimeFrom">The date time from.</param>
        /// <param name="dateTimeTo">The date time to.</param>
        /// <param name="queueOperation">The queue operation.</param>
        /// <param name="queueStatus">The queue status.</param>
        /// <param name="patientIdentifier">The patient identifier.</param>
        /// <param name="user">The user responsible for the action.</param>
        /// <returns>
        /// List of queued operations and indicator of success or failure.
        /// </returns>
        [OperationContract]
        QueuedOperationStatus GetQueuedOperationList(DateTime dateTimeFrom, DateTime dateTimeTo, IList<QueueOperation> queueOperation, IList<QueueStatus> queueStatus, IList<PatientIdentifierBase> patientIdentifier, UserDetails user);

        /// <summary>
        /// Returns the Document and Attachments required to display the requested view.
        /// </summary>
        /// <param name="patientIdentifier">An identifier for the patient.</param>
        /// <param name="user">Details of the person responsible for the action.</param>
        /// <param name="request">The view request, which must be one of the concrete view types under the declared base type.</param>
        /// <returns>DocumentResponse will also pass back success and error messages</returns>
        [OperationContract]
        ViewResponse GetView(PatientIdentifierBase patientIdentifier, UserDetails user, ViewRequestBase request);

        /// <summary>
        /// For a given MRN, State Patient Identifier, PCEHR Data Store PatientMasterId or a validated IHI,
        /// checks whether the patient has advertised the existence of his/her PCEHR,
        /// whether access has been gained, and whether access can be gained without a code.
        /// </summary>
        /// <param name="patientIdentifier">An identifier of the patient (MRN, State ID, PatientMasterId or IHI)</param>
        /// <param name="dateOfBirth">The patient's date of birth.</param>
        /// <param name="user">The health provider individual, interactive user or authorised employee responsible for this action</param>
        /// <returns>Response containing operation success indicator, whether advertised, and access status</returns>
        [OperationContract]
        DoesPcehrExistResponse IsPcehrAdvertised(PatientIdentifierBase patientIdentifier, DateTime dateOfBirth, UserDetails user);

        /// <summary>
        /// Lists the documents uploaded for a specific patient in a defined hospital
        /// </summary>
        /// <param name="user">Information to identify the person responsible for this action</param>
        /// <param name="sourceSystemSetId">The source system set identifier.</param>
        /// <param name="patientIdentifier">The local, state or national patient identifier</param>
        /// <returns>
        /// Response containing the a list of the admitted or discharged episodes for the patient with in the specified number of days.,
        /// </returns>
        [OperationContract]
        LocalClinicalDocumentListResponse<PatientIdentifierBase> ListUploadedDocuments(UserDetails user, string sourceSystemSetId, PatientIdentifierBase patientIdentifier);

        /// <summary>
        /// This operation looks up a patient record in the current facility (for
        /// advertised status) and disclosure facility (for disclosed status),
        /// gets a validated IHI for the patient, and optionally refreshes the
        /// advertised status from the PCEHR system.
        /// </summary>
        /// <param name="user">Information to identify the person responsible for this action</param>
        /// <param name="patientIdentifier">An identifier of the patient and the
        /// current facility.</param>
        /// <param name="disclosureFacility">The Hospital Code and Hospital Code
        /// System of the facility whose healthcare provider organisation records
        /// the PCEHR disclosure status for the patient.</param>
        /// <param name="forceRefresh">The option of always refreshing the
        /// participation status, only if not advertised, or never refreshing the
        /// participation status.</param>
        /// <returns>Participation status for the requested patient.</returns>
        [OperationContract]
        PatientParticipationResponse RefreshPatientParticipationStatus(
            UserDetails user,
            PatientIdentifierBase patientIdentifier,
            HospitalIdentifier disclosureFacility,
            ForceRefreshOption forceRefresh);

        /// <summary>
        /// Removes the specified discharge summary from the national PCEHR repository.
        /// </summary>
        /// <param name="patientIdentifier">An identifier of the patient (MRN, State ID, PatientMasterId or IHI)</param>
        /// <param name="admissionDate">Episode Admission Date</param>
        /// <param name="documentSetId">Clinical Document Set Id</param>
        /// <param name="reason">Reason for Removal</param>
        /// <param name="user">User details</param>
        /// <param name="auditInformation">Additional audit information to store</param>
        /// <returns>Response indicating that the request was queued or the reason why not.</returns>
        [OperationContract]
        HipsResponse Remove(PatientIdentifierBase patientIdentifier, DateTime admissionDate, string documentSetId, RemovalReason reason, UserDetails user, byte[] auditInformation);

        /// <summary>
        /// Returns the Document and Attachments required to display the requested document.
        /// </summary>
        /// <param name="patientIdentifier">An identifier for the patient.</param>
        /// <param name="user">Details of the person responsible for the action.</param>
        /// <param name="request">Identifiers required to locate the document.</param>
        /// <returns>DocumentResponse will also pass back success and error messages</returns>
        [OperationContract]
        DocumentResponse RetrieveDocument(PatientIdentifierBase patientIdentifier, UserDetails user, DocumentRequest request);

        /// <summary>
        /// Uploads a CDA document to the PCEHR National Repository, optionally superseding a previously-uploaded document.
        /// This takes in the XML document and any attachments and does the packaging itself.
        /// </summary>
        /// <param name="cdaDocument">The CDA root XML document.</param>
        /// <param name="patientIdentifier">The patient identifier at hospital, state, PDS or national level.</param>
        /// <param name="user">The user.</param>
        /// <param name="attachments">The attachments</param>
        /// <param name="admissionDate">The admission date</param>
        /// <param name="documentFormatCode">The document format code (template ID).</param>
        /// <returns>Response indicating that the request was queued or the reason why not.</returns>
        [OperationContract]
        HipsResponse UploadOrSupersedeDocument(byte[] cdaDocument, PatientIdentifierBase patientIdentifier, UserDetails user, Attachment[] attachments, DateTime admissionDate, string documentFormatCode = null);
    }
}